@@ -4,34 +4,30 @@ use super::geometry::Tri;
 
 use crate::linear_algebra::math::dot;
 
-#[derive(Clone, Debug)]
+/// A collection of inter-connected triangles.
+#[derive(Clone, Debug, Default)]
 pub struct Mesh {
-    pub ids: Vec<i32>,
-    pub vertices: Vec<f32>,
-}
-
-impl Default for Mesh {
-    fn default() -> Self {
-        Self {
-            ids: Vec::new(),
-            vertices: Vec::new(),
-        }
-    }
+    ids: Vec<usize>,
+    vertices: Vec<f32>,
 }
 
 impl Mesh {
-    pub fn from(other_meshes: Vec<&Self>) -> Self {
+    /// Create a new Mesh from a list of ids and vertices.
+    pub const fn from(ids: Vec<usize>, vertices: Vec<f32>) -> Self {
+        Self {
+            ids,
+            vertices,
+        }
+    }
+
+    /// Combine different meshes all into one
+    pub fn combine(other_meshes: Vec<&Self>) -> Self {
         let mut id_offset = 0;
 
-        let mut n_ids = 0;
-        let mut n_vertices = 0;
+        let n_ids = other_meshes.iter().map(|mesh| mesh.ids.len()).sum();
+        let n_vertices = other_meshes.iter().map(|mesh| mesh.vertices.len()).sum();
 
-        for m in &other_meshes {
-            n_ids += m.ids.len();
-            n_vertices += m.vertices.len();
-        }
-
-        let mut ids: Vec<i32> = Vec::with_capacity(n_ids);
+        let mut ids: Vec<usize> = Vec::with_capacity(n_ids);
         let mut vertices: Vec<f32> = Vec::with_capacity(n_vertices);
 
         for m in other_meshes {
@@ -43,50 +39,16 @@ impl Mesh {
                 vertices.push(*vertex);
             }
 
-            id_offset += (m.vertices.len() / 3) as i32;
+            id_offset += m.vertices.len() / 3;
         }
 
-        // remove duplicate vertices and their ids
-        // let mut i = 0;
-        // let mut dups = 0;
-
-        // loop {
-        //     let id = (ids[i] * 3) as usize;
-        //     match vertices[..id].iter().position(|&x| x == vertices[id + 0]) {
-        //         Some(j) => {
-        //             if vertices[j + 1] == vertices[id + 1] && vertices[j + 2] == vertices[id + 2] {
-        //                 // dups += 1;
-        //                 n_ids -= 3;
-
-        //                 vertices.remove(id + 0);
-        //                 vertices.remove(id + 1);
-        //                 vertices.remove(id + 2);
-
-        //                 ids.remove(i + 0);
-        //                 ids.remove(i + 1);
-        //                 ids.remove(i + 2);
-
-        //                 for l in i..n_ids {
-        //                     ids[l] -= 3;
-        //                 }
-        //             }
-        //         }
-        //         None => {}
-        //     };
-
-        //     i += 3;
-        //     if i >= n_ids {
-        //         break;
-        //     }
-        // }
-        // println!("Removed {} duplicate vertices!", dups);
-
         Self {
             ids,
             vertices,
         }
     }
 
+    /// Transform the mesh by the given matrix.
     pub fn transform(&self, a: Mat3A) -> Self {
         debug_assert_eq!(self.vertices.len() % 3, 0);
         debug_assert_eq!(self.ids.len() % 3, 0);
@@ -114,6 +76,7 @@ impl Mesh {
         }
     }
 
+    /// Translate the mesh by the given vector.
     pub fn translate(&self, p: Vec3A) -> Self {
         debug_assert_eq!(self.vertices.len() % 3, 0);
 
@@ -125,6 +88,7 @@ impl Mesh {
         }
     }
 
+    /// Convert the mesh to a list of triangles.
     #[rustfmt::skip]
     pub fn to_triangles(&self) -> Vec<Tri> {
         let n = self.ids.len() / 3;
@@ -133,7 +97,7 @@ impl Mesh {
         for i in 0..n {
             triangles.push(Tri::default());
             for j in 0..3 {
-                let id = (self.ids[i * 3 + j] * 3) as usize;
+                let id = self.ids[i * 3 + j] * 3;
                 triangles[i].p[j].x = self.vertices[id    ] as f32;
                 triangles[i].p[j].y = self.vertices[id + 1] as f32;
                 triangles[i].p[j].z = self.vertices[id + 2] as f32;
